ifndef BUILDING
DIRS_MK := $(CURDIR)/dirs.mk
OS_MK := $(CURDIR)/os.mk
endif

ifdef version
ifdef release
RELEASE_BUILD := 1
endif
endif

ifndef version
export version := 0.0
endif

ifndef release
export release := 1test
endif

CFLAGS :=
CPPFLAGS :=
DEPFLAGS :=
PREPROCESS_FLAGS :=

-include $(OS_MK)

export GLOBAL_INCLUDE_DIRS =
add_include_paths = $(addprefix -I,$(1))
ifndef SHOW_INCLUDES
CPPFLAGS += $${GLOBAL_INCLUDE_DIRS}
else
CPPFLAGS += $(GLOBAL_INCLUDE_DIRS)
endif

dep_suffixes := .c

include $(DIRS_MK)

build_root := $(strip $(build_root))
build_dirs := $(strip $(build_dirs))
common_dirs := $(strip $(common_dirs))
source_dirs := $(strip $(source_dirs))

rooted_build_dirs := $(addprefix $(build_root)/,$(build_dirs))

ifndef BUILDING

relative_common_dirs := $(filter-out /% ~%,$(common_dirs))
common_dirs := $(foreach dir,$(common_dirs),$(if $(findstring $(dir),$(relative_common_dirs)),$(CURDIR)/$(dir),$(dir)))

all_source_dirs := $(CURDIR) $(common_dirs)
endif

all_includes.mk := $(foreach srcdir,$(source_dirs),$(patsubst %,%/$(srcdir)/include.mk,$(all_source_dirs)))

-include $(all_includes.mk)

VPATH := $(all_source_dirs)

ifdef RELEASE_BUILD
targets += $(release_targets)
endif

ifndef BUILDING

active_build_dirs := $(addprefix $(build_root)/,$(filter $(build_dirs),$(MAKECMDGOALS)))
active_build_dirs += $(filter $(addprefix $(build_root)/,$(build_dirs)),$(MAKECMDGOALS))

ifeq ($(strip $(active_build_dirs)),)
active_build_dirs := $(rooted_build_dirs)
endif

all: $(active_build_dirs)

$(rooted_build_dirs):%:%/Makefile $(prep_targets)
	@$(MAKE) -C $* $(filter-out $(build_dirs) $(rooted_build_dirs),$(MAKECMDGOALS))  

$(addsuffix /Makefile,$(rooted_build_dirs)):%/Makefile:Makefile $(CONFIG_MK) $(MAIN_MK) $(DIRS_MK)
	@mkdir -p $(dir $@) && cat $< | \
	$(AWK) 'BEGIN { print "CONFIG_MK := $(CONFIG_MK)"; \
	                print "MAIN_MK := $(MAIN_MK)"; \
		        print "DIRS_MK := $(DIRS_MK)"; \
                        print "OS_MK := $(OS_MK)"; \
	                print "all_source_dirs := $(all_source_dirs)"; \
		        print "build_dirs := ./"; \
		        print "building_in := $(patsubst $(build_root)/%/,%,$(dir $@))"; \
	                print "BUILDING := 1\n"; } \
	        !/(export (CONFIG_MK)|(MAIN_MK) :=.*)|(build_dirs :=.*)/ { print; }' > $@ 

prep_build_dirs: $(addsuffix /Makefile,$(active_build_dirs)) $(prep_targets)

lint $(CLEAN_TARGETS) $(build_dirs) install rpms: all

distclean:
	-rm -rf $(patsubst %/.,%,$(active_build_dirs))

.PHONY: prep_build_dirs lint $(rooted_build_dirs) $(build_dirs)

else

cmdline_change := $(shell \
  $(ECHO) 'MAKEOVERRIDES: $(MAKEOVERRIDES)' | $(DIFF) -q make.cmdline - > /dev/null 2>&1; \
  if [ $$? != 0 ]; then \
    $(ECHO) 'MAKEOVERRIDES: $(MAKEOVERRIDES)' > make.cmdline; \
    $(TOUCH) Makefile; \
  fi)

Makefile : $(wildcard $(all_includes.mk))
	@touch $@

lint: $(sources:.c=.ln)

.PHONY: lint

clean:
	@find . -name '*.$(OBJ_SUFFIX)' | xargs rm -f	
	@find . -name '*.i' | xargs rm -f	
	@find . -name '*.ln' | xargs rm -f	
	@find . -name '*.ph' | xargs rm -f	
	@find . -name '*.stripped' | xargs rm -f	
	@find . -name '*.mangled' | xargs rm -f	
	@find . -name '*.mapped' | xargs rm -f	
	@find . -name '*.cmdline' | $(SED) 's/^.*\/make\.cmdline//' | xargs rm -f

depclean: 
	@find . -name '*.d' -type f | xargs rm -f

realclean: clean depclean
	@find . -name 'make.cmdline' | xargs rm -f

distclean:
	@rm -rf *

ifeq ($(strip $(filter $(CLEAN_TARGETS),$(MAKECMDGOALS))),)
GLOBAL_INCLUDE_DIRS += $(call add_include_paths,$(all_source_dirs))
-include $(filter %.d,$(foreach suffix,$(dep_suffixes),$(patsubst %$(suffix),%.d,$(sources))))
endif

all: $(targets)

endif

.PHONY: all $(CLEAN_TARGETS)

PYTHON := /usr/bin/python
$(OS_MK): $(MAIN_MK)
	@echo "# This file is automatically generated -- do not edit" > $@
	@case `uname -s` in \
					Linux ) \
						echo -e "\
OS :=LINUX\n\
SHELL := /bin/sh\n\
CC := /usr/bin/gcc\n\
CPP := /usr/bin/cpp\n\
CXX := /usr/bin/g++\n\
STRIP := /usr/bin/strip\n\
AWK := /bin/awk\n\
PYTHON := /usr/bin/python\n\
PYTHONCOMP := $(PYTHON) $(shell $(PYTHON) -c "import distutils.sysconfig; print distutils.sysconfig.get_config_vars()['LIBP']")/py_compile.py\n\
SED := /bin/sed\n\
GREP := /bin/grep\n\
KILL := /bin/kill\n\
ECHO := /bin/echo\n\
ECHOE := /bin/echo -e\n\
CHMOD := /bin/chmod \n\
CP := /bin/cp\n\
MV := /bin/mv\n\
CUT := $(shell which cut)\n\
LN := /bin/ln\n\
CAT := /bin/cat\n\
DIFF := /usr/bin/diff\n\
TOUCH := /bin/touch\n\
MKDIR :=/bin/mkdir\n\
INSTALL := /usr/bin/install\n\
FIND := /usr/bin/find\n\
XARGS := /usr/bin/xargs\n\
RM := /bin/rm\n\
RPM := /bin/rpm\n\
RPMBUILD := $(shell if [ -x /usr/bin/rpmbuild ]; then echo /usr/bin/rpmbuild; \
                                                 else echo /bin/rpm; fi)\n\
MD5SUM := /usr/bin/md5sum\n\
LINT1 := /usr/bin/lclint\n\
LINTFLAGS := -weak -booltype bool_t -booltrue True -boolfalse False \
             -relaxquals -warnposix\n\
ECHON := /bin/echo -n\n\
OBJ_SUFFIX :=o\n\
LD := /usr/bin/ld\n\
DEPFLAGS += -M\n\
CFLAGS += -Wall -Werror\n\
PREPROCESS_FLAGS := -E -P\n\
XMLLINT := /usr/bin/xmllint\n\
XSLTPROC := /usr/bin/xsltproc\n\
XGETTEXT := /usr/bin/xgettext\n\
MSGCAT := /usr/bin/msgcat\n\
MSGFMT := /usr/bin/msgfmt\n\
MSGINIT := /usr/bin/msginit\n\
MSGEXEC := /usr/bin/msgexec\n\
MSGCONV := /usr/bin/msgconv\n\
MSGFILTER := /usr/bin/msgfilter\n\
MSGMERGE := /usr/bin/msgmerge\n\
MSGATTRIB := /usr/bin/msgattrib\n\
TEST := /usr/bin/test\n\
M4 := /usr/bin/m4\n\
TRUE := /bin/true\n\
DATE := /bin/date\n\
CPPFLAGS += -DENSIM_LINUX" >> $@;; \
					SunOS) \
						/usr/bin/echo "\
OS :=SOLARIS\n\
SHELL := /usr/bin/sh\n\
CC := /opt/SUNWspro/bin/cc\n\
LINT1 := /opt/SUNWspro/SC5.0/bin/lint\n\
LINT2 := /opt/SUNWspro/SC5.0/bin/lint2n\n\
LINTFLAGS += -err=warn -errchk -errfmt=simple -Ncheck -Nlevel=4\n\
LINT2FLAGS += -err=warn -errchk -errfmt=simple -Ncheck -Nlevel=4 -Xarch=v9\n\
AWK := /usr/local/bin/awk\n\
SED := /usr/bin/sed\n\
KILL := /usr/bin/kill\n\
ECHO := /usr/ucb/echo\n\
ECHOE := /usr/bin/echo\n\
CP := /usr/bin/cp\n\
CUT := /usr/bin/cut\n\
CAT := /usr/bin/cat\n\
DIFF := /usr/bin/diff\n\
TOUCH := /usr/bin/touch\n\
ECHON := /usr/ucb/echo -n\n\
OBJ_SUFFIX :=o\n\
LD := /usr/ccs/bin/ld\n\
DEPFLAGS += -xM1\n\
PREPROCESS_FLAGS := -P\n\
CFLAGS += -errwarn=%all -v\n\
CPPFLAGS += -DENSIM_SOLARIS" >> $@;; \
					*) \
						echo -e "\
OS :=NT\n\
CC := cl\n\
OBJ_SUFFIX :=.obj\n\
LD := link\n\
DEPFLAGS += -MM\n\
CPPFLAGS += -DENSIM_NT" >> $@;; \
				   esac

ifdef RELEASE_BUILD
CPPFLAGS += -DRELEASE_BUILD
else
CFLAGS += -g
endif

.SUFFIXES:
.SUFFIXES: .d .c .$(OBJ_SUFFIX) .s .i .ln .ph .stripped .mangled .mapped

.DELETE_ON_ERROR:
.PRECIOUS: %.d %.c %.$(OBJ_SUFFIX) %.s %.i %.ln %.ph %.stripped %.mangled %.mapped %.map

.PHONY: all $(CLEAN_TARGETS)

define cmdcheck
  $(if $(filter-out Makefile,$?),
      @$2 
      @$(ECHO) '$(subst ','\'',$1)' > $@.cmdline
      $1
      ,
      @$(ECHO) '$(subst ','\'',$1)' | $(DIFF) $@.cmdline - > /dev/null 2>&1; \
      if [ $$? != 0 ]; then \
        $2 \
        $(if $(filter-out @%,$(word 1,$1)),$(ECHO) '$(subst ','\'',$(subst $${GLOBAL_INCLUDE_DIRS},\$${GLOBAL_INCLUDE_DIRS},$1))';) \
        $(ECHO) '$(subst ','\'',$1)" > $@.cmdline; \
        $(patsubst @%,%,$(word 1,$1)) $(wordlist 2,$(words $1),$1); \
      else \
        exit 0; \
      fi;)
endef


define print-header
$(ECHOE) '#\n# $(build_info) $(dir $@)$(notdir $<) ---> $@\n#' && \
$(MKDIR) -m 755 -p $(dir $@);
endef


% : Makefile


ifndef EXTRA_DEPENDS
ifndef STRIP_EXTRA_DEPENDS
STRIP_EXTRA_DEPENDS := $(CAT)
endif
else
STRIP_EXTRA_DEPENDS := $(CAT)
endif

%.d : %.c
	$(call cmdcheck,$(subst $(comma),$(comma),@set -e;  \
         $(CC) $(DEPFLAGS) $(CPPFLAGS) $< \
	 | $(SED) 's/\($(notdir $*)\)\.o[ :]*/$(subst /,\/,$(dir $*))\1.$(OBJ_SUFFIX) $(subst /,\/,$@) : Makefile /g' | $(STRIP_EXTRA_DEPENDS) > $@ && \
		[ -s $@ ] || rm -f $@ && $(ECHON) .),mkdir -p $(dir $@);)

ifndef EXPLICIT_COMPILE_STAGES
 ifeq ($(OS),NT)
%.obj : %.c Makefile
	$(call cmdcheck,$(subst $(comma),$(comma),$(CC) -c /Fo $@ $(CPPFLAGS) $(CFLAGS) $(ENSIM_FLAGS) $<),$(print-header))
 else 
%.o : %.c Makefile
	$(call cmdcheck,$(subst $(comma),$(comma),$(CC) -c -o $@ $(CPPFLAGS) $(CFLAGS) $(ENSIM_FLAGS) '-DENSIMDEBUG_FILE="$(dir $@)$(notdir $<)"' $<),$(print-header))
 endif
endif


% : %.c

% : %.o
	$(call cmdcheck,$(subst $(comma),$(comma),$(LD) -o $@ $(LDFLAGS) $^),$(print-header))
%.o : %.s
	$(call cmdcheck,$(subst $(comma),$(comma),$(CC) -c -o $@ $(ASFLAGS) $<),$(print-header))

%.s : %.i
	$(call cmdcheck,$(subst $(comma),$(comma),$(CC) -S -fverbose-asm -o $@ $(CFLAGS) $<),$(print-header))

%.i : %.c Makefile
	$(call cmdcheck,$(subst $(comma),$(comma),$(CC) $(PREPROCESS_FLAGS) -o $@ $(CPPFLAGS) $(ENSIM_FLAGS) $<),\
	  $(print-header))

%.ln : %.c Makefile
	$(call cmdcheck,$(subst $(comma),$(comma),$(LINT1) -C$@ $(LINTFLAGS) $(CPPFLAGS) $(ENSIM_FLAGS) $<),$(print-header))

# XXX why are we stripping the directory names below ?!

%.ph : %.h Makefile
	$(call cmdcheck,$(subst $(comma),$(comma),h2ph -d $(dir $@) < $< > $@; \
	  perl -pi -e "next if /require '(linux|asm|bits)\//; \
		s:require '[^/]*/:require ':; \
		s:require 'stdint.ph':require 'broken_h2ph_stdint.ph':" $@),$(print-header))

%.stripped : % %.no_strip util/stripper
	$(call cmdcheck,$(subst $(comma),$(comma),$(CP) $< $@; $(filter %/stripper,$^) $@ $(word 2,$^) || rm $@),$(print-header))

%.mangled : % %.no_mangle util/mangler
	$(call cmdcheck,$(subst $(comma),$(comma),$(CP) $< $@; $(filter %/mangler,$^) $@ $(word 2,$^) $*.map \
		$(firstword $(subst ., ,$(notdir $@))) || rm $@),$(print-header))

%.mapped : %
	$(call cmdcheck,$(subst $(comma),$(comma),objcopy `cat $(filter %.map,$(^:.mangled=.map)) | awk '{printf "--redefine-sym=%s=%s ", $$2, $$1 }'` $< $@),$(print-header))

%.prof : %.o util/build_prof
	$(word 2,$^) $< > $@

#endif
