#!/bin/bash

cmd=$0
script_dir=`dirname $0`

do_clean=0
do_register=0

while getopts crhd OPT; do
  case $OPT in
    c)
      do_clean=1
      ;;
    r)
      do_register=1
      ;;
    d)
      do_delete=1
      ;;
    h)
      echo "
Usage: $cmd [-c] [-r] [-h] [-d]

   -c : clears database
   -d : deletes instance information, but does not clear entire database
   -r : registers DSPs
   -h : show this message
"
      ;;
  esac
done

if [ x"$do_clean" = x"1" -o x"$do_delete" = x"1" ]; then
    service mysqld stop
    for dir in `ls /var/lib/mysql | egrep -v '^(mysql|admin_appl)'`; do
        rm -rf /var/lib/mysql/$dir
    done
    for site in `ls -d /home/virtual/site[0-9]*`; do
        for dir in `ls $site/fst/var/lib/mysql | egrep -v '^(mysql|admin_appl)'`; do
            rm -rf $site/fst/var/lib/mysql/$dir
        done
    done
    rm -f /etc/virtualhosting/mappings/mysql.dbmap
    service mysqld start
    for line in `echo "select shortname, version, dapiversion, dapirelease from deployables" | psql appldb | tail +3 | grep '|' | tr -d ' '`; do
	shortname=`echo $line | cut -f1 -d'|'`
	version=`echo $line | cut -f2 -d'|'`
	dapiversion=`echo $line | cut -f3 -d'|'`
	dapirelease=`echo $line | cut -f4 -d'|'`
        # Be aggressive in removing older tables and sequences
        if [ x"$do_clean" = x"1" ]; then
            psql appldb <<EOF
DROP TABLE "${shortname}-${version}-${dapiversion}-${dapirelease}";
DROP SEQUENCE "${shortname}-${version}-${dapiversion}-${dapirelease}_seq";
DROP TABLE "${shortname}-${version}-${dapiversion}";
DROP SEQUENCE "${shortname}-${version}-${dapiversion}_seq";
EOF
        else
            psql appldb <<EOF
DELETE FROM "${shortname}-${version}-${dapiversion}";
EOF
        fi
    done
    if [ x"$do_clean" = x"1" ]; then
        psql appldb <<EOF
DROP TABLE deployables_plans;
DROP TABLE deployables_sites;
DROP TABLE deployables_upgrades;
DROP TABLE deployables;
EOF
    fi
fi

if [ x"$do_register" = x"1" ]; then
    /usr/local/bin/InitializeAddonSupport -v 1.0 -r 1
    for dir in `find /usr/lib/ensim/deployables -type d -maxdepth 1 | grep -- '-'`; do
        dsp=`basename $dir`
        ndashes=`echo $dsp | awk 'BEGIN {FS = "-";} {print NF;}'`
        shortname=`echo $dsp | cut -f1-$(($ndashes - 1)) -d'-'`
        version=`echo $dsp | cut -f$ndashes -d'-'`
	ver_rel=`xsltproc $script_dir/ver_rel.xsl /usr/lib/ensim/deployables/$dsp/lib/dsp.xml`
        dapiversion=`echo $ver_rel | cut -f1 -d' '`
        dapirelease=`echo $ver_rel | cut -f2 -d' '`
	echo "Registering $shortname $version (${dapiversion}-${dapirelease})"
        /usr/local/bin/RegisterAddon -s $shortname -v $version -V $dapiversion -R $dapirelease -r
    done
fi 
