import deployables_lib as dl

dl.load_msgs_module("phpnuke-6.8", symtab = globals())

dir_moves = [('html', ''), ('', 'archive')]
pat = '^\s*\$(?P<key>\S+)\s*=\s*(?P<value>["\'].*[\'"])\s*;\s*$'
sub = '$%(key)s = %(value)s;'

def install(site, site_conf, instance_conf, msgs):

    # XXX need to change sitekey in config.php
    vars = [('dbhost', '127.0.0.1'),
            ('dbuname', site_conf['mysql']['dbaseadmin']),
            ('dbpass',  instance_conf['dbpass']),
            ('dbname',  instance_conf['dbname'])]

    actions = ['resolve_location', 'check_disk_quota', 'check_mysql_enabled', 
	       'check_pass', '', 'db_create', 'db_probe', 'archive_unpack',
               'run_sql_file', 'edit_conf_file', 'exec_php_script',
               'handle_security_mode', 'access_url']
    args_dict = {
        'archive_unpack':       { 'dir_moves': dir_moves },
        'run_sql_file':         { 'file': 'archive/sql/nuke.sql' },
        'edit_conf_file':       { 'file': 'config.php', 'lang': 'php',
                                  'pat': pat, 'sub': sub, 'vars': vars },
        'exec_php_script':      {
            'script': 'admin.php',
            'args': { 'name': instance_conf['instance_admin'],
                      'url': 'http://%s/' % site_conf['siteinfo']['domain'],
                      'email': instance_conf['instance_admin_email'],
                      'pwd': instance_conf['instance_pass1'],
                      'user_new': '1',
                      'fop': 'create_first' }},
        'handle_security_mode': { 'files': ['config.php'] },
        }
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf,
                instance_conf, old_instance_conf, msgs):

    vars = [('dbuname', site_conf['mysql']['dbaseadmin'],
                    old_site_conf['mysql']['dbaseadmin']),
            ('dbname', instance_conf['dbname'],
                   old_instance_conf['dbname'])]
    actions = ['handle_url_loc', 'resolve_location', '',
               'archive_unpack', 'handle_username_change',
               'handle_db_changes', 'edit_conf_file', 'handle_security_mode']

    args_dict = {
        'archive_unpack': { 'dir_moves': dir_moves,
			    'fileinfo' :
			    {'excluded_files' : ['config.php']}},
        'edit_conf_file': { 'file': 'config.php', 'lang': 'php',
                            'pat': pat, 'sub': sub, 'vars': vars },
        'handle_security_mode': { 'files': ['config.php'] },
        }
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)     

def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves)

def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])

    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path', 'instance_admin_email'])
    return instance
