import deployables_lib as dl

dl.load_msgs_module("phpopenchat-3.0.0b2", symtab = globals())

dir_moves = [('phpopenchat-3.0.0b2', '')]
file_mods = [{'path' : 'tmp/', 'perms' : 01777, 'create' : None}]
pat = "^\s*define\('(?P<key>[^']+)'\s*,\s*(?P<value>'.*')\)\s*;(?P<comment>\s*/\*.*)?$"
sub = "define('%(key)s',%(value)s);%(comment)s"

def install(site, site_conf, instance_conf, msgs):

    actions = ['check_disk_quota', 'check_mysql_enabled', 'check_pass', '',
               'resolve_location', 'db_create', 'db_probe', 'archive_unpack',
               'run_sql_file']
    args_dict = {
        'archive_unpack':       { 'dir_moves': dir_moves, 
				  'file_mods': file_mods },
        'run_sql_file':         { 'file': 'db.schema' }
        }
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)
    
    if msgs.get_severity() != msgs.ERROR:
        sql_stmt1 = """
            UPDATE poc_user_account SET password = '%(instance_pass1)s'
            WHERE user = 'operator'
            """ % instance_conf
        vars = [('DATABASE_HOST',       '127.0.0.1'),
                ('DATABASE_USER',       site_conf['mysql']['dbaseadmin']),
                ('DATABASE_PASSWORD',   instance_conf['dbpass']),
                ('DATABASE_TABLESPACE', instance_conf['dbname']),
                ('TMPDIR', 
                 (instance_conf['install_loc'] + '/tmp', dl.MODSTR_JAIL),
                 '/tmp')]

        actions = ['edit_conf_file', 'handle_security_mode', 'exec_mysql']
        args_dict = {'edit_conf_file': { 'file': 'config.inc.php',
                                         'lang': 'php', 'pat': pat,
                                         'sub': sub, 'vars': vars },
                     'handle_security_mode': { 'files': ['config.inc.php'] },
                     'exec_mysql' : {'sql' : sql_stmt1}
                     }
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)
     

def reconfigure(context, site, site_conf, old_site_conf,
                instance_conf, old_instance_conf, msgs):

    vars = [('DATABASE_USER',       site_conf['mysql']['dbaseadmin'],
                                old_site_conf['mysql']['dbaseadmin']),
            ('DATABASE_TABLESPACE', instance_conf['dbname'],
                                old_instance_conf['dbname'])]
    actions = ['handle_url_loc', 'resolve_location', '',
               'archive_unpack', 'handle_username_change',
               'handle_db_changes', 'edit_conf_file', 'handle_security_mode']

    args_dict = {
        'archive_unpack': { 'dir_moves': dir_moves },
        'edit_conf_file': { 'file': 'config.inc.php', 'lang': 'php',
                            'pat': pat, 'sub': sub, 'vars': vars },
        'handle_security_mode': { 'files': ['config.inc.php'] },
        }
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)     

def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves)

def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])

    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path'])
    return instance
