import deployables_lib as dl

dl.load_msgs_module("postnuke-0.7.2.3", symtab = globals())

dir_moves = [('postnuke-phoenix-0.7.2.3/html', ''),
             ('postnuke-phoenix-0.7.2.3', 'archive')]

def install(site, site_conf, instance_conf, msgs):

    actions = ['check_disk_quota', 'check_mysql_enabled', 'check_pass', '',
               'resolve_location', 'db_create', 'db_probe', 'archive_unpack']
    args_dict = {'archive_unpack':       { 'dir_moves': dir_moves }}
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        actions = ['exec_php_script', 'exec_php_script2',
                   'handle_security_mode', 'access_url']
        args_dict = {
            'exec_php_script' : {
                'script' : 'install.php',
                'args' : {'op' : 'Start',
                          'currentlang' : 'eng',
                          'dbhost' : '127.0.0.1',
                          'dbuname' : site_conf['mysql']['dbaseadmin'],
                          'dbpass' : instance_conf['dbpass'],
                          'dbname' : instance_conf['dbname'],
                          'prefix' : '',
                          'dbtype' : 'mysql',
                          'intranet' : '0'}},
            'exec_php_script2' : {
                'script' : 'install.php',
                'args' : {'op' : 'Set Login',
                          'currentlang' : 'eng',
                          'dbhost' : '127.0.0.1',
                          'dbuname' : site_conf['mysql']['dbaseadmin'],
                          'dbpass' : instance_conf['dbpass'],
                          'dbname' : instance_conf['dbname'],
                          'prefix' : '',
                          'dbtype' : 'mysql',
                          'intranet' : '0',
                          'aid' : instance_conf['instance_admin'],
                          'name' : instance_conf['instance_admin_name'],
                          'pwd' : instance_conf['instance_pass1'],
                          'repeatpwd' : instance_conf['instance_pass1'],
                          'email' : instance_conf['instance_admin_email'],
                          'url' : instance_conf['url_loc']}},
            'handle_security_mode': { 'files': ['config.php'] },
            }
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf,
                instance_conf, old_instance_conf, msgs):

    pat = "^\s*\$pnconfig\['(?P<key>\S+)'\]\s*=\s*(?P<value>'.*')\s*;\s*$"
    sub = "$pnconfig['%(key)s'] = %(value)s;"
    vars = [('dbname', instance_conf['dbname'],
                   old_instance_conf['dbname']),
            ('dbuname', site_conf['mysql']['dbaseadmin'],
                    old_site_conf['mysql']['dbaseadmin'])]

    actions = ['handle_url_loc', 'resolve_location', '',
               'archive_unpack', 'handle_username_change',
               'handle_db_changes', 'edit_conf_file', 'handle_security_mode']
    args_dict = {
        'archive_unpack': { 'dir_moves': dir_moves },
        'edit_conf_file': { 'file': 'config.php', 'lang': 'php',
                            'pat': pat, 'sub': sub, 'vars': vars },
        'handle_security_mode': { 'files': ['config.php'] },
        }
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)     

def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves)

def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])
    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path'])
    return instance
