# This variable is a list of all of the DSP directories that do not
# have their own dsp.spec file. These DSPs will use the generic dsp.spec
# file defined at the top level. Those DSPs that have their own dsp.spec
# files will use their own.

ifdef BUILDING

define extract_lang
$(word $(words $(subst -,$(space),$1)),$(subst -,$(space),$1))
endef

define extract_dsp
$(patsubst %-$(word $(words $(subst -,$(space),$1)),$(subst -,$(space),$1)),%,$1)
endef

generic_rpm_spec_dsps := $(foreach src_dir,$(all_source_dirs),$(foreach dsp_dir,$(dsp_dirs),$(if $(wildcard $(src_dir)/$(dsp_dir)/dsp.spec),,$(dsp_dir))))

generic_locale_spec_dsps := $(foreach src_dir,$(all_source_dirs),$(foreach dsp_dir,$(dsp_dirs),$(if $(wildcard $(src_dir)/$(dsp_dir)/locale.spec),,$(dsp_dir))))

$(addsuffix /dsp.spec,$(generic_rpm_spec_dsps)) : rpmbuild/dsp.spec
	$(call cmdcheck,$(INSTALL) -m 0755 $< $@,$(print-header))

$(addsuffix /locale.spec,$(foreach trans,$(translations),$(addsuffix /$(trans),$(generic_locale_spec_dsps)))) : rpmbuild/locale.spec
	$(call cmdcheck,$(INSTALL) -m 0755 $< $@,$(print-header))

locale_rpms := $(foreach trans,$(translations),$(foreach dsp,$(dsps),RPMS/noarch/$(dsp)-$(trans)-locale-rpm))

$(dsp_rpms) : RPMS/i386/%-rpm :
	$(call cmdcheck, \
          $(FIND) $(CURDIR)/$($*_directory)/INSTALL -name "*.cmdline" | $(XARGS) $(RM) -f && \
          $(MKDIR) -p SRPMS BUILD && $(RPMBUILD) -bb $(filter %/dsp.spec,$^) \
          --define "VERSTR $($*_rpm_version)" \
	  --define "RELSTR $($*_rpm_release)" \
          --define "DEPLOYABLES_DIR /$(deployables_dir)" \
          --define "DSP_NAME $($*_name)" --define "DSP_VERSION $($*_version)" \
          --define "DSP_DIR $($*_directory)" \
          --define "DSP_NAME_VER $($*_name_ver)" \
	  --define "DSP_SHORTNAME $($*_shortname)" \
          --buildroot $(CURDIR)/$($*_directory)/INSTALL \
          --define "_topdir $(CURDIR)" && $(TOUCH) $@, $(print-header))

$(locale_rpms) : RPMS/noarch/%-locale-rpm :
	$(call cmdcheck, \
	  $(FIND) $(CURDIR)/$($*_directory) -name "*.cmdline" | $(XARGS) $(RM) -f && \
	  $(FIND) $(CURDIR)/$($*_directory) -name "*.untranslated" | $(XARGS) $(RM) -f && \
	  $(MKDIR) -p SRPMS BUILD && $(RPMBUILD) -bb $(filter %.spec,$^) \
	  --define "VERSTR $($(call extract_dsp,$*)_rpm_version)" \
          --define "RELSTR $($(call extract_dsp,$*)_rpm_release)" \
          --define "DEPLOYABLES_DIR /$(deployables_dir)" \
          --define "DSP_NAME $($(call extract_dsp,$*)_name)" \
	  --define "DSP_VERSION $($(call extract_dsp,$*)_version)" \
          --define "DSP_DIR $($(call extract_dsp,$*)_directory)" \
          --define "DSP_NAME_VER $($(call extract_dsp,$*)_name_ver)" \
          --define "DSP_SHORTNAME $($(call extract_dsp,$*)_shortname)" \
          --define "DSP_LANG $(call extract_lang,$*)" \
          --buildroot $(CURDIR)/$($(call extract_dsp,$*)_directory)/$(call extract_lang,$*)/INSTALL \
          --define "_topdir $(CURDIR)" && $(TOUCH) $@, $(print-header))

$(dsp_installs) : %_install : 
	@$(ECHOE) '#\n# $(build_info) Installing $*\n#' && \
	$(MKDIR) -p /$(deployables_dir) && \
	$(RM) -rf /$(deployables_dir)/$* && \
	$(CP) -al $($*_directory)/INSTALL/$(deployables_dir)/$* /$(deployables_dir)/ && \
	$(foreach trans,$(translations),\
                  $(CP) -al $($*_directory)/$(trans)/INSTALL/$(deployables_dir)/$* /$(deployables_dir)/ &&) \
	$(TRUE)


# Generate a listing of the RPMs that belong to this group.
RPMS/i386/group%.manifest : 
	@$(MKDIR) -p $(dir $@) && $(RM) -f $@ && \
	$(foreach dsp,$(group$*),$(ECHO) "ensim-$(dsp)-$($(dsp)_rpm_version)-$($(dsp)_rpm_release).i386.rpm" >> $@ &&) /bin/true

manifests := $(addprefix RPMS/i386/,$(addsuffix .manifest,$(groups)))

sources += $(manifests)

rpms : $(dsp_rpms) $(locale_rpms)
$(manifests) : rpms
.PHONY : xml manifests
manifests : $(manifests)
xml: rpms
	@for rpm in `ls RPMS/i386/*.rpm RPMS/noarch/*.rpm`; do \
	   md5sum=`md5sum $$rpm | cut -f1 -d' '` && \
	   rpm -qp --qf "<package>\n\t<shortname>%{NAME}</shortname>\n\t<version>%{VERSION}</version>\n\t<release>%{RELEASE}</release>\n\t<arch>%{ARCH}</arch>\n\t<os>Linux</os>\n\t<blob>`basename $$rpm`</blob>\n\t<md5sum>$$md5sum</md5sum>\n</package>\n" $$rpm > $${rpm}.xml; done

install : $(dsp_installs)

.PHONY: edit dsp_install
edit : 
	/usr/local/bin/EditVirtDomain -c scriptsmgr,$(subst $(space),,$(foreach dsp,$(dsps),$(dsp)=1$(comma))on) $(if $(domain),$(domain),test.com) || $(TRUE)

dsp_install:
	$(foreach dsp,$(dsps),$(ECHO) "Installing $(dsp)..." && \
                              /usr/local/bin/InstallAddon $(if $(domain),$(domain),test.com) $(dsp) \
	  $(if $(filter $(dsp),invboard-1.1.2 invboard-1.2),--accept_lic=1 ) \
	  $(if $(filter $(dsp),gallery-1.3.4 invboard-1.1.2 oscommerce-2.2ms2 \
	                       phpbb-2.0.6 phpnuke-6.9 phpopenchat-3.0.0b3 \
                               postnuke-0.7.2.6 b2-0.6.2.1 xoops-2.0.3 \
                               phplinks-2.1.3.1 phprojekt-4.0.4 invboard-1.2), \
	                       --instance_pass1=ensim --instance_pass2=ensim ) \
	  $(if $(filter $(dsp),invboard-1.1.2 oscommerce-2.2ms2 \
	                       phpbb-2.0.6 phpnuke-6.9 phpopenchat-3.0.0b3 \
                               postnuke-0.7.2.6 b2-0.6.2.1 xoops-2.0.3 \
                               phplinks-2.1.3.1 phprojekt-4.0.4 \
                               webcalendar-0.9.42 invboard-1.2), \
	                       --dbpass=ensim ) $(other_dsp_args) \
	  $(if $(bool),$(bool),&&)) $(TRUE)
endif
