import deployables_lib as dl
import urlparse
import shutil
import os

dl.load_msgs_module("DSP_SHORTNAME-DSP_VERSION", symtab = globals())

dir_moves = [('sohoadmin', '')]
file_mods = [{'path' : 'sohoadmin/', 'create' : '', 'delete' : ''},
             {'path' : 'sohoadmin/tmp_content/', 'create' : ''}]
secure_files = [('config/isp.conf.php')]
instance_patches = []

pat = "^\s*\$(?P<key>[^= ]+)\s*=\s*(?P<value>'?.*'?)\s*;(?P<comment>\s*(/\*|//).*)?\s*$"
sub = "$%(key)s = %(value)s;%(comment)s"

def install(site, site_conf, instance_conf, msgs):

    actions = ['resolve_location', 'check_disk_quota', 'check_mysql_enabled', 
	       'check_pass', 'check_install_loc_owner', '']
    args_dict = {}
    dl.do_install_actions(site, site_conf, instance_conf, msgs,
                          globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        sql_stmt1 = ["""
            UPDATE b2users SET user_pass = '%(instance_pass1)s',
            user_login = '%(instance_admin)s'
            WHERE user_login = 'admin'
            """ % instance_conf]
        actions = ['db_create', 'db_probe', 'archive_unpack', 'check_index',
                   'exec_php_script',
                   'handle_security_mode', 'access_url']
        args_dict = {
            'archive_unpack':       {'dir_moves': dir_moves,
                                     'file_mods' : file_mods},
            'handle_security_mode': {'files': secure_files},
            'exec_php_script' : {
                 'script' : 'setup.php',
                 'args' : {'STEP' : 'SETUP_NOW',
                           'SETUP_ROOT' : 
                               (instance_conf['install_loc'], dl.MODSTR_JAIL),
                           'SETUP_IP' : instance_conf['url_loc'],
                           'SETUP_DBSERVER' : '127.0.0.1',
                           'SETUP_DBUN' : site_conf['mysql']['dbaseadmin'],
                           'SETUP_DBPW' : instance_conf['dbpass'],
                           'SETUP_DBNAME' : instance_conf['dbname'],
                           'SERVER_SOFTWARE' : 'Apache/Linux',
                           'OSTYPE' : 'Linux'},
                 'success_re' : '.*Location: index.php'},
            'exec_mysql' : {'sql' : sql_stmt1},
            }
        dl.do_install_actions(site, site_conf, instance_conf, msgs,
                              globals(), actions, args_dict)

def reconfigure(context, site, site_conf, old_site_conf, instance_conf,
                old_instance_conf, msgs):

    actions = ['handle_url_loc', 'resolve_location', '', 'archive_unpack',
               'handle_username_change', 'handle_db_changes']
    if context != dl.CONTEXT_SITE_EDIT and instance_conf.get('dbpass'):
        actions.insert(0, 'db_probe')
    args_dict = {
            'archive_unpack':       { 'dir_moves': dir_moves,
                                      'fileinfo' :
                                      {'excluded_files' :
                                       ['config/isp.conf.php']}},
            }
    dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                              instance_conf, old_instance_conf, msgs,
                              globals(), actions, args_dict)
    if msgs.get_severity() != msgs.ERROR:
        old_root = "%s%s" % (dl.site_fs_jail(site, old_site_conf),
                             instance_conf['install_loc'])
        new_root = "%s%s" % (dl.site_fs_jail(site, site_conf),
                             instance_conf['install_loc'])
        old_cgi = "%s%s%s" % (dl.site_fs_jail(site, old_site_conf),
                              instance_conf['install_loc'],
                              '/sohoadmin/tmp_content')
        new_cgi = "%s%s%s" % (dl.site_fs_jail(site, site_conf),
                              instance_conf['install_loc'],
                              '/sohoadmin/tmp_content')
        old_url = old_instance_conf['url_loc'][len('http://'):]
        new_url = instance_conf['url_loc'][len('http://'):]
        vars = [('doc_root', new_root, old_root),
                ('cgi_bin', new_cgi, old_cgi),
                ('this_ip', new_url, old_url),
                ('db_un', site_conf['mysql']['dbaseadmin'],
                   old_site_conf['mysql']['dbaseadmin']),
                ('db_name', instance_conf['dbname'], old_instance_conf['dbname'])]
        if context != dl.CONTEXT_SITE_EDIT and instance_conf.get('dbpass'):
            vars.append(('db_pw', instance_conf['dbpass']))
        actions2 = ['edit_conf_file', 'apply_patch', 'handle_security_mode']
        args_dict = {
            'handle_security_mode': { 'files': secure_files },
            'edit_conf_file' : {'file' : 'config/isp.conf.php', 'lang' : 'php',
                                'pat' : pat, 'sub' : sub, 'vars' : vars},
            'apply_patch' : {'patches' : instance_patches}
            }
        dl.do_reconfigure_actions(context, site, site_conf, old_site_conf,
                                  instance_conf, old_instance_conf, msgs,
                                  globals(), actions2, args_dict)
        
        
def remove(site, site_conf, instance_conf, msgs):
    dl.db_drop(site, site_conf, instance_conf)
    dl.install_loc_cleanup(site, site_conf, instance_conf, dir_moves)
    
def get_instance_configuration(context, site, site_conf, instance_conf,
                               old_instance_conf, locale, msgs):
    instance = dl.dsp_create_instance(instance_conf['shortname'],
                                      instance_conf['version'])
    dl.dsp_fill_instance(context, instance, site, site_conf,
                         instance_conf, old_instance_conf,
                         ['dbname', 'username', 'url_net', 'url_path',
                          'domain_to_path', 'instance_admin_email'])
    dl.enable_patches(instance, instance_patches)
    return instance
