ifdef BUILDING

# If you need to add another .po file (e.g. fr_FR), add the file to the
# translations directory as fr_FR.po, and add fr_FR to the list of translations
# below. The translations will be automatically distributed to the proper files,
# and any outstanding untranslated strings will be gathered back to the file
# build/translations/untranslated_msgs.<locale>.po .

translations := en_GB de_DE es_ES

translations/translations.d : make_utils/gen_trans_targets translations/include.mk dirs.mk
	$(call cmdcheck,dsp_dirs="$(dsp_dirs)" translations="$(translations)" $< > $@,$(print-header))
-include translations/translations.d

translations/en_GB.po : translations/unified_msgs.po
	$(call cmdcheck,$(MSGCONV) -i $< | $(MSGFILTER) -o $@ --keep-header --force-po sed -e 's/\(.*\)/en_US(\1)/',$(print-header))

translations/en_US.po : translations/unified_msgs.po 
	$(call cmdcheck,$(MSGCAT) -o $@ $<,$(print-header))

# Generate a single .po file with all unique messages; this is what will be
# handed to translators.
translations/unified_msgs.po : 
	$(call cmdcheck,$(MSGCAT) -o $@ -t utf-8 --use-first --no-location $^,$(print-header))

$(po_targets) : 
	$(call cmdcheck,$(MSGMERGE) $(filter %.po,$^) $(filter %/msgs.pot,$^) | $(MSGATTRIB) -o $@ --no-obsolete > $@,$(print-header))

targets += translations/unified_msgs.po
targets += $(po_targets)
targets += $(mo_targets)

%/dsp.list : %/dsp.xml make_utils/dsp_strings.xslt util/dsp_options.xsl
	$(call cmdcheck,$(XSLTPROC) $(filter %/util/dsp_options.xsl,$^) $< | $(XSLTPROC) -o $@ $(filter %/make_utils/dsp_strings.xslt,$^) -,$(print-header))

%/lib/msgs.pot : util/common_msgs.list
	$(call cmdcheck, $(XGETTEXT) --no-location -s -o $@ -L C -a --copyright-holder="Ensim Corporation" $(filter %/msgs.list,$^) $(filter %/dsp.list,$^) $(filter %/util/common_msgs.list,$^), $(print-header))

# The project-id script should be part of the gettext RPM but is
# somehow missing from the rh73 version. When me move to rh8 or later
# on the build machines, we can drop the GETTEXTLIBDIR variable and
# the project-id script from the make_utils directory.

%/locale/en_US/LC_MESSAGES/msgs.po.template : %/msgs.pot make_utils/project-id
	$(call cmdcheck, LANG=en_US.utf8 GETTEXTLIBDIR=$(dir $(filter %/make_utils/project-id,$^)) $(MSGINIT) -i $< -o $@ --no-translator >/dev/null, $(print-header))

%/locale/en_US/LC_MESSAGES/msgs.po : %/locale/en_US/LC_MESSAGES/msgs.po.template
	$(call cmdcheck, $(SED) -e 's/^# English translations for PACKAGE package./# English translations for the Ensim Deployable $($(word 1,$(subst /, ,$*))_name) $($(word 1,$(subst /, ,$*))_version) package./' \
                                -e 's/^# This file is distributed under the same license as the PACKAGE package./# This file is distributed under the same license as the $(word 1,$(subst /, ,$*))-$(version)-$(release) package./' \
                                -e 's/^"Project-Id-Version: PACKAGE VERSION\\n"/"Project-Id-Version: $(word 1,$(subst /, ,$*)) $(version)-$(release)\\n"/' $< > $@, \
          $(print-header))

%.mo : %.po
	$(call cmdcheck,$(MSGFMT) -c -o $@ $<,$(print-header))

%.po.fuzzy.tmp : %.po
	$(call cmdcheck,$(MSGATTRIB) --force-po -o $@ --only-fuzzy $<,$(print-header))

%.po.untranslated.tmp : %.po
	$(call cmdcheck,$(MSGATTRIB) --force-po -o $@ --untranslated $<,$(print-header))

%.po.untranslated : %.po.untranslated.tmp %.po.fuzzy.tmp
	$(call cmdcheck,$(MSGCAT) --force-po $^ | $(MSGFILTER) --keep-header --force-po -o $@  sed -e 's/.*//' 2>/dev/null,$(print-header))

missing_translations := $(foreach trans,$(filter-out en_GB,$(translations)),translations/untranslated_msgs.$(trans).po)
targets += $(missing_translations)

$(missing_translations) : translations/untranslated_msgs.%.po :
	$(call cmdcheck,$(MSGCAT) -o $@ $^,$(print-header))

# Targets for building just the po and/or mo files.
# If you want to generate the localization files for a subset
# of the available translations, you can use the command
# make targets=mo_targets translations="<trans1> <trans2> ..."
# where you would replace <trans<n>> with the shortname of
# the desired language; e.g. en_GB

mo_targets : $(mo_targets)
po_targets : $(po_targets)

.PHONY: msgmerge msgadd

ifneq ($(filter msgmerge msgadd,$(targets)),)
ifeq ($(words ($filter msgmerge msgadd,$(targets))),2)
foo := $(shell $(ECHO) "You may only merge or add one language at a time" && $(KILL) -INT $$PPID)
endif
ifeq ($(lang),)
foo := $(shell $(ECHO) "You must specify a language (e.g. lang=en_GB)" && $(KILL) -INT $$PPID)
endif
ifneq ($(filter msgmerge,$(targets)),)
ifeq ($(newstrings),)
foo := $(shell $(ECHO) "You must specify a new strings file (e.g. newstrings=en_GB.new)" && $(KILL) -INT $$PPID)
endif
ifneq ($(filter $(lang),$(newstrings)),)
foo := $(shell $(ECHO) "Please rename the new strings file to something other than $(lang)" && $(KILL) -INT $$PPID)
endif
endif
endif

msgadd : translations/$(lang).po translations/$(lang).po.new
	$(call cmdcheck,$(CP) $(filter %.po.new,$^) $<,$(print-header))

translations/$(lang).po.new : translations/$(lang).po
	$(call cmdcheck,$(MSGCAT) -o $@ --strict -i -s --no-location $^, $(print-header))

msgmerge : translations/$(lang).po translations/$(lang).po.merge \
		translations/$(lang).po.merge.fuzzy \
		translations/$(lang).po.merge.untrans
	$(call cmdcheck,($(TEST) ! -e $(filter %.po.merge.fuzzy,$^) || \
			 ($(ECHO) "Fuzzy strings found" && exit 1)) && \
			($(TEST) ! -e $(filter %.po.merge.untrans,$^) || \
			 ($(ECHO) "Untranslated strings found" && exit 1)) && \
			$(CP) $(filter %.po.merge,$^) $<,$(print-header))

translations/$(lang).po.merge : translations/$(lang).po $(newstrings)
	$(call cmdcheck,$(MSGCAT) -o $@ --strict -i -s --no-location $^, $(print-header))

translations/$(lang).po.merge.test : translations/$(lang).po.merge $(newstrings)
	$(call cmdcheck,$(MSGMERGE) -v -o - $^ | \
			$(MSGATTRIB) --no-obsolete -o $@ -i -,$(print-header))

translations/$(lang).po.merge.fuzzy : translations/$(lang).po.merge.test
	$(call cmdcheck,$(MSGATTRIB) -o $@ --only-fuzzy $<, $(print-header))

translations/$(lang).po.merge.untrans : translations/$(lang).po.merge.test
	$(call cmdcheck,$(MSGATTRIB) -o $@ --untranslated $<, $(print-header))

endif
