<?php //-*-php-*-
/*   ********************************************************************   **
**   Copyright notice                                                       **
**                                                                          **
**   (c) 1995-2003 PHPOpenChat Development Team                             **
**   http://phpopenchat.sourceforge.net/                                    **
**                                                                          **
**   All rights reserved                                                    **
**                                                                          **
**   This script is part of the PHPOpenChat project. The PHPOpenChat        **
**   project is free software; you can redistribute it and/or modify        **
**   it under the terms of the GNU General Public License as published by   **
**   the Free Software Foundation; either version 2 of the License, or      **
**   (at your option) any later version.                                    **
**                                                                          **
**   The GNU General Public License can be found at                         **
**   http://www.gnu.org/copyleft/gpl.html.                                  **
**   A copy is found in the textfile GPL and important notices to the       **
**   license from the team is found in the textfile LICENSE distributed     **
**   with these scripts.                                                    **
**                                                                          **
**   This script is distributed in the hope that it will be useful,         **
**   but WITHOUT ANY WARRANTY; without even the implied warranty of         **
**   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          **
**   GNU General Public License for more details.                           **
**                                                                          **
**   This copyright notice MUST APPEAR in all copies of the script!         **
**   ********************************************************************   */

/**
* Get default values
*/
require_once(POC_BASE.'/config.inc.php');
require_once(POC_INCLUDE_PATH.'/class.Logger.inc');

/**
 * Class Line holds lines written by chatters
 *
 * @todo rewrite of method filter_input_buffer() based on preg_replace()
 * @author Frerk Meyer <frerk@meychern.de>
 * @author Michael Oertel <michael@ortelius.de>
 * @access  public
 * @version $Id: class.Line.inc,v 1.42.2.1 2003/07/24 20:11:24 letreo Exp $
 */
class Line {

  /**
  * chatter object represents a person taking part in a chat session
  *
  * @var object
  */
  var $chatter = null;

  /**
  * recipient object represents a person receiving a chat message
  *
  * @var object
  */
  var $recipient = null;

  /**
  * string said by chatter
  *
  * @var string
  */
  var $said = '';

  /**
  * true if line is whispered
  *
  * @var boolean
  */
  var $whispered = false;
  
  /**
  * true if line is approved by a moderator
  *
  * @var boolean
  */
  var $approved = false;
  
  /**
  * contains an irc-command found in the line
  *
  * @var array
  */
  var $irc_command = array();
  
  /**
  * contains accepted mime types for private images
  *
  * @var array
  */
  var $mime_types = array();
  
  /**
  * counts smileys within a line
  *
  * @var integer
  */
  var $smiley_count = 0;

  /**
  * marks a line as invitation message
  *
  * @var boolean
  */
  var $invitationMsg = false;

  /**
  * private window flag
  *
  * @var boolean
  */
  var $in_private_window = false;

  /**
  * sender busy flag
  *
  * @var boolean
  */
  var $sender_busy = false;

  /**
  * @var string
  */
  var $join = '';

  /**
  * @var string
  */
  var $leave = '';

  /**
  * @var boolean
  */
  var $info = false;

  /**
   * Constructor for class Line
   *
   * @param object $chatter chatter object represents a person taking part in a chat session
   * @param string $said string said by chatter
   * @param object recipient of the text line
   */
  function Line($chatter, $said)
  {
    // check $chatter - should be of class Chatter
    // check $said - should be of type string
    $this->chatter   = $chatter;
    $this->said      = $said;
  }
  
  function _mk_smiley_img_tag($smiley)
  {
    return '<img src="'.$_SESSION['template']->get_tmpl_web_path().'/images/icons/smileys/'.$smiley.'.gif" align="middle" border="0" />';
  }
  
  /**
   * Getter for chatter object
   *
   * @return object
   */
  function get_chatter()
  {
    return $this->chatter;
  }
  
  /**
   * @ignore
   */
  function get_sender()
  {
    return $this->get_chatter();
  }
  
  /**
   * Getter for recipient object
   *
   * @return object
   */
  function get_recipient()
  {
    return $this->recipient;
  }

  /**
   * Setter for recipient object
   *
   * @param object
   */
  function set_recipient( $recipient )
  {
    if(!is_object( $recipient) ) die('Recipient MUST be an object!');
    $this->recipient = $recipient;
  }

  /**
   * Setter for invitationMsg
   *
   * @param boolean
   */
  function set_invitationMsg( $bool = true )
  {
      $this->invitationMsg = $bool;
  }

  /**
   * Provides the invitationMsg flag
   *
   * @return boolean
   */
  function get_invitationMsg()
  {
      return $this->invitationMsg;
  }
  function is_invitationMsg()
  {
      return $this->get_invitationMsg();
  }

  /**
   * Setter for sender object
   *
   * @param object
   */
  function set_sender( $sender )
  {
      $this->chatter = $sender;
      return true;
  }

  /**
   * Setter for sender busy flag
   *
   * @param boolean
   */
  function set_sender_busy( $bool = true )
  {
      $this->sender_busy = $bool;
  }

  /**
   * Setter for info flag
   *
   * @param boolean
   */
  function set_info( $bool = true )
  {
      $this->info = $bool;
  }

  /**
   * Getter for info flag
   *
   * @return boolean
   */
  function is_info()
  {
      return $this->info;
  }

  /**
   * Getter for sender busy flag
   *
   * @return boolean
   */
  function get_sender_busy()
  {
      return $this->sender_busy;
  }
  /** alias for get_sender_busy() */
  function is_sender_busy(){return $this->get_sender_busy();}
  
  /**
   * alias
   * @ignore
   */
  function set_chatter( $sender )
  {
      return set_sender($sender);
  }

  /**
   * Getter for said string
   *
   * @return string
   */
  function get_said()
  {
    return $this->said;
  }
  
  /**
   * Setter for said string
   *
   * @param string
   */
  function set_said( $said )
  {
    $this->said = $said;
  }

  /**
  * Getter for chatter object
  *
  * @return object
  */
  function get_whispered()
  {
    return $this->whispered;
  }
  function is_whispered(){return $this->get_whispered();}//alias
  
  /**
   * Setter for var $whispered
   *
   * @param boolean
   * @return boolean
   */
  function set_whispered( $bool = true )
  {
    $this->whispered = $bool;
  }
  
  /**
   * Sets approved flag
   *
   * @param boolean
   */
  function set_approved( $bool = true )
  {
    $this->approved = $bool;
  }
  
  /**
   * Gets approved flag
   *
   * @return boolean
   */
  function get_approved()
  {
    return $this->approved;
  }
  
  /**
   * Gets the irc-command of line
   *
   * @return string
   */
  function get_irc_command()
  {
    return $this->irc_command;
  }
  
  /**
   * Provides the nickname of the chatter who is logging in.
   *
   * @return string
   */
  function get_login()
  {
    return $this->login;
  }
  
  /**
   * Provides the nickname of the chatter who is leaving the chat.
   *
   * @return string
   */
  function get_leave()
  {
    return $this->leave;
  }
  
  /**
   * Sets the nickname of the chatter who is logging in.
   *
   * @return string
   */
  function set_login($login)
  {
    $this->login = $login;
  }
  
  /**
   * Sets the nickname of the chatter who is leaving the chat.
   *
   * @return string
   */
  function set_leave($leave)
  {
    $this->leave = $leave;
  }

  function is_loginMsg()
  {
    return ( $this->login != '' );
  }

  function is_leavingMsg()
  {
    return ( $this->leave != '' );
  }
  
  /**
   * Sets accepted mime types for private images
   *
   * @param array
   */
  function set_accepted_mime_types($types)
  {
    $this->mime_types = $types;
  }
  
  /**
   * Returns a file name for the given smiley
   *
   * @access private
   * @param string
   * @return string
   */
  function _smiley_mapping($code)
    {
    if( $this->smiley_count >= MAX_SMILEYS_PER_LINE )
      return $code;
    else 
      $this->smiley_count++;
    
    if( $code == ':me' )
    {
      $sender = $this->get_sender();
      $smiley_dir  = '/images/icons/chatter';
      for ($i=0;$i<count($this->mime_types);$i++)
      {
        preg_match('#image/[x\-]?(.*)#',$this->mime_types[$i], $parts);
        $file_extension = $parts[1];
        $smiley_path = $smiley_dir.'/'.strtolower($sender->get_nick()).'.'.$file_extension;
        if( file_exists($_SESSION['template']->get_tmpl_sys_path().$smiley_path) )
          return '<img src="'.$_SESSION['template']->get_tmpl_web_path().$smiley_path.'" align="middle" alt="'.$_SESSION['translator']->out('PRIVATE_IMAGE').'" />';
      }
      
      return $code;
    }

    if( $code == ':you' )
    {
      $recipient = $this->get_recipient();
      $smiley_dir  = '/images/icons/chatter';
      for ($i=0;$i<count($this->mime_types);$i++)
      {
        preg_match('#image/[x\-]?(.*)#',$this->mime_types[$i], $parts);
        $file_extension = $parts[1];
        $smiley_path = $smiley_dir.'/'.strtolower($recipient->get_nick()).'.'.$file_extension;
        if( file_exists($_SESSION['template']->get_tmpl_sys_path().$smiley_path) )
          return '<img src="'.$_SESSION['template']->get_tmpl_web_path().$smiley_path.'" align="middle" alt="'.$_SESSION['translator']->out('PRIVATE_IMAGE').'" />';
      }
      
      return $code;
    }

    switch ($code)
    {
      case ':-)': return $this->_mk_smiley_img_tag('smile');
      case ':)' : return $this->_mk_smiley_img_tag('smile');
      case ':-(': return $this->_mk_smiley_img_tag('frown');
      case ':(':  return $this->_mk_smiley_img_tag('frown');
      case ':,-(':return $this->_mk_smiley_img_tag('crying');
      case ':,(': return $this->_mk_smiley_img_tag('crying');
      case ':-p': return $this->_mk_smiley_img_tag('tongue');
      case ':p':  return $this->_mk_smiley_img_tag('tongue');
      case ':-x': return $this->_mk_smiley_img_tag('kiss');
      case ':-D': return $this->_mk_smiley_img_tag('biggrin');
      case ':D':  return $this->_mk_smiley_img_tag('biggrin');
      case ':-]': return $this->_mk_smiley_img_tag('evillaugh');
      case ':]':  return $this->_mk_smiley_img_tag('evillaugh');
      case '=:-)':return $this->_mk_smiley_img_tag('punk');
      case ':-)=':return $this->_mk_smiley_img_tag('beard');
      case 'B-))':return $this->_mk_smiley_img_tag('poc');
      case 'B-)': return $this->_mk_smiley_img_tag('poc');
      case '(/)': return $this->_mk_smiley_img_tag('denied');
      case '(:(=':return $this->_mk_smiley_img_tag('ghost');
      case '~==': return $this->_mk_smiley_img_tag('candle');
      case '~o':  return $this->_mk_smiley_img_tag('bomb');
      case '~--': return $this->_mk_smiley_img_tag('dynamite');
      case ':#D': return $this->_mk_smiley_img_tag('sail');
      case ':-Q': return $this->_mk_smiley_img_tag('smoke');
      case ':[=]':return $this->_mk_smiley_img_tag('trash');
      case '8)':  return $this->_mk_smiley_img_tag('frog');

      case ':toffline:':
      case ':sun:':
      case ':sail:':
      case ':rose:':
      case ':candle:':
      case ':denied:':
      case ':dynamite:':
      case ':rain:':
      case ':frog:':
      case ':massa:':
      case ':love:':
      case ':kisslipps:':
      case ':flower:':
      case ':cloud:':
      case ':kiss:':
      case ':eating:':
      case ':cool:':
      case ':rolleyes:':
      case ':yawn:':
      case ':crying:':
      case ':angel:':
      case ':wow:':
      case ':finger:':
      case ':hehe:':
      case ':evillaugh:':
      case ':fairy:':
      case ':lovestory:':
      case ':verysad:':
      case ':help:':
      case ':repuke:':
      case ':scream:':
      case ':male:':
      case ':mail:':
      case ':female:':
      case ':rambo:':
      case ':nono:':
      case ':stoned:':
      case ':thumbsup:':
      case ':toilet:':
      case ':smoky:':
      case ':kidding:':
      case ':lol:':
      case ':question:':
      case ':hi:':
      case ':phoneme:':
      case ':confused:':
      case ':borg:':
      case ':apresent:':
      case ':alarm:':
      case ':dazzler:':
      case ':tombstone:':
      case ':director:':
      case ':birthday:': return $this->_mk_smiley_img_tag( preg_replace('/:/','',$code) );
      default: return $code;
    }
  }

  /**
   * Filters irc-commands in $said
   *
   * @return null
   */
  function filter_irc()
  {
    //filter IRC like commands
    if( preg_match ( '#^/([a-z]+) ?([^ ]*) ?([0-9]*).*$#i', $this->said, $irc_command ) && $irc_command[1] != '')
    {
      $this->irc_command[] = $irc_command[1];
      $this->irc_command[] = $irc_command[2];
      $this->irc_command[] = $irc_command[3];

      return null;
    }
  }
  
  /**
   * Filters icons, links, etc. in $said
   *
   * @return null
   */
  function filter_buffer_input()
  {
    if( count($this->irc_command) > 0 )
      return null;
    //filter urls
    elseif ( preg_match ( '#www\.|https?://|ftp://|.+@.+\..+#', $this->said) )
    {
      $pieces = explode(' ',$this->said);
      reset($pieces);
      do{
        if( preg_match ('#www\.|https?://|ftp://#',current($pieces)) )
        {
          $pieces[key($pieces)] = eregi_replace("((((https?://([^ :@]*(:[^ :@]*)?@)?)|(ftp://([^ :@]*(:[^ :@]*)?@)?))?[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}(:[0-9]{1,5})?(/[^][ <>@()]*)?)|(((https?://([^ :@]*(:[^ :@]*)?@)?)|(ftp://([^ :@]*(:[^ :@]*)?@)?)|(www\.))[^] <>/@()]+\.[a-z]{2,}(:[0-9]{1,5})?(/[^][ <>@()]*)?))",
            "<a class=\"linkWithinOutputFrame\" href=\"jump.php?url=\\0\" target=\"_blank\">\\0</a>",current($pieces));
        }
        elseif( preg_match('/.*@.*\..*/',current($pieces)) )
        {
          $pieces[key($pieces)] = eregi_replace("[^][ <>@():]+@[^][ <>@():]+\.[a-z]{2,}","<a class=\"linkWithinOutputFrame\" href=\"mailto:\\0\">\\0</a>",current($pieces));
        }
      }while(next($pieces));
      $this->said = implode($pieces,' ');
      
      return null;
    }
    
    //insert smiley-images if configured
    if( SMILEYS_AS_IMAGES )
      $this->said = preg_replace('/(=?[:|B|\(|~|8]\w*:?[^ |\<|\w]*[D|x|Q]?)/e', '\$this->_smiley_mapping("\1")' ,$this->said );

    //insert private-images if configured
    if( ALLOW_PRIVATE_IMAGES )
      $this->said = preg_replace('/(:[me|you]*)/e', '\$this->_smiley_mapping("\1")' ,$this->said );
    
    $this->said = preg_replace( "/\#{3}([^#]*)\#{3}/e", '\1', $this->said );

    return null;
  }

  /**
   * Filters 'says to', 'leaves us', ... messages within output frame
   *
   * used in getlines.php only
   *
   * @param object Translator
   * @return null
   */
  function filter_buffer_output()
  {
    $this->said = preg_replace( "/\#{3}([^#]*)\#{3}/e", "\$_SESSION['translator']->out('\\1')", $this->said );

    return null;
  }

  /**
  * Returns a formatted Line
  *
  * @see getlines.php
  * @return string
  */
  function out()
  {
    $sender = $this->chatter;
    $recipient = $this->recipient;
    
    //whispering to somebody?!
    if( $this->get_whispered()
    && is_object($recipient)
    && $recipient->get_nick() == $_SESSION['translator']->out('EVERYBODY')
    )
      $this->set_whispered(false);
    $italic = ( $this->is_whispered() )? ';font-style:italic':'';
    $line  = '<div style="color: #'.$sender->get_color().$italic.'">'.HTML_BEFORE_LINE;
    unset($italic);
    $line .= $_SESSION['chat']->get_grade_icon( $sender );
    $gender  = '';
    if( SHOW_GENDER_ICON )
      $gender = ($sender->get_gender())?'<img src="'.$_SESSION['template']->get_tmpl_web_path().'/images/icons/'.$sender->get_gender().'.gif" width="8" height="8" alt="'.$_SESSION['translator']->out('GENDER').'" /> ':'<img src="'.$_SESSION['template']->get_tmpl_web_path().'/images/dot_clear.gif" width="8" height="8" alt="" /> ';
    $line .= $gender.'<span>'.$sender->get_nick().'</span>&nbsp;';
    if( $this->get_whispered() )
      $line .= $_SESSION['translator']->out('WHISPERS_TO');
    else
      $line .= $_SESSION['translator']->out('SAYS_TO');
    
    if( is_object($recipient) )
      $line .= '&nbsp;<span>'.$recipient->get_nick().'</span>:&nbsp;';
    
    $this->filter_buffer_output();
    $line .= $this->get_said().HTML_AFTER_LINE.'</div>';

    unset($sender);
    unset($recipient);
    
    return preg_replace('#</#', '<\\/', $line);
  }
  
  /**
  * Provides the private window flag 
  *
  * @return boolean
  */
  function in_private_window()
  {
    return $this->in_private_window;
  }
  
  /**
  * Sets the private window flag 
  *
  * @param boolean
  */
  function set_in_private_window( $bool = true )
  {
    $this->in_private_window = $bool;
  }
  
  /**
  * Test class Line
  * 
  * Call like this: Line::test();
  */
  function test() {
    $line = new Line("Fritz","Hello World!");
    print "Line test, ";
    print $line->get_chatter();
    print " said: ";
    print $line->get_said();
    print "<br>\n";
  }
}
?>
