<?php
// $Id: form.php,v 1.6 2003/02/14 01:24:08 buennagel Exp $
//  ------------------------------------------------------------------------ //
//                XOOPS - PHP Content Management System                      //
//                    Copyright (c) 2000 XOOPS.org                           //
//                       <http://www.xoops.org/>                             //
//  ------------------------------------------------------------------------ //
//  This program is free software; you can redistribute it and/or modify     //
//  it under the terms of the GNU General Public License as published by     //
//  the Free Software Foundation; either version 2 of the License, or        //
//  (at your option) any later version.                                      //
//                                                                           //
//  You may not change or alter any portion of this comment or credits       //
//  of supporting developers from this source code or any supporting         //
//  source code which is considered copyrighted (c) material of the          //
//  original comment or credit authors.                                      //
//                                                                           //
//  This program is distributed in the hope that it will be useful,          //
//  but WITHOUT ANY WARRANTY; without even the implied warranty of           //
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the            //
//  GNU General Public License for more details.                             //
//                                                                           //
//  You should have received a copy of the GNU General Public License        //
//  along with this program; if not, write to the Free Software              //
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA //
//  ------------------------------------------------------------------------ //
// Author: Kazumi Ono (AKA onokazu)                                          //
// URL: http://www.myweb.ne.jp/, http://www.xoops.org/, http://jp.xoops.org/ //
// Project: The XOOPS Project                                                //
// ------------------------------------------------------------------------- //
// public abstruct
/**
 * 
 * 
 * @package     kernel
 * @subpackage  form
 * 
 * @author	    Kazumi Ono	<onokazu@xoops.org>
 * @copyright	copyright (c) 2000-2003 XOOPS.org
 */


/**
 * Abstract base class for forms
 * 
 * @author	Kazumi Ono	<onokazu@xoops.org>
 * @copyright	copyright (c) 2000-2003 XOOPS.org
 * 
 * @package     kernel
 * @subpackage  form
 */
class XoopsForm {
	/**#@+
	 * @access  private
	 */
    /** 
     * "action" attribute for the html form
     * @var string  
     */
	var $_action;

	/**
     * "method" attribute for the form.
	 * @var string  
	 */
	var $_method;

	/**
     * "name" attribute of the form
	 * @var string  
	 */
	var $_name;

	/**
     * title for the form
	 * @var string  
	 */
	var $_title;

	/**
     * array of {@link XoopsFormElement} objects
     * @var  array   
     */
	var $_elements = array();

	/**
     * extra information for the <form> tag
	 * @var string  
	 */
	var $_extra;

	/**
     * required elements
	 * @var array   
	 */
	var $_required = array();
    
    /**#@-*/

	/**
	 * constructor
	 * 
     * @param	string  $title  title of the form
     * @param	string  $name   "name" attribute for the <form> tag
     * @param	string  $action "action" attribute for the <form> tag
     * @param   string  $method "method" attribute for the <form> tag
	 */
	function XoopsForm($title, $name, $action, $method="post"){
		$this->_title = $title;
		$this->_name = $name;
		$this->_action = $action;
		$this->_method = $method;
	}

	/**
	 * return the title of the form
     * 
     * @return	string
	 */
	function getTitle(){
		return $this->_title;
	}

	/**
	 * get the "name" attribute for the <form> tag
     * 
	 * @return	string
	 */
	function getName(){
		return $this->_name;
	}

	/**
	 * get the "action" attribute for the <form> tag
	 * 
     * @return	string
	 */
	function getAction(){
		return $this->_action;
	}

	/**
	 * get the "method" attribute for the <form> tag
	 * 
     * @return	string
	 */
	function getMethod(){
		return $this->_method;
	}

	/**
	 * Add an element to the form
	 * 
     * @param	object  &$formElement    reference to a {@link XoopsFormElement}
     * @param	bool    $required       is this a "required" element?
	 */
	function addElement(&$formElement, $required=false){
		$this->_elements[] =& $formElement;
		if ($required) {
			$this->_required[] =& $formElement;
		}
	}

	/**
	 * get the array of the forms elements
	 * 
     * @return	array   array of {@link XoopsFormElement}s
	 */
	function &getElements(){
		return $this->_elements;
	}

	/**
	 * set the extra attributes for the <form> tag
	 * 
     * @param	string  $extra  extra attributes for the <form> tag
	 */
	function setExtra($extra){
		$this->_extra = " ".$extra;
	}

	/**
	 * get the extra attributes for the <form> tag
	 * 
     * @return	string
	 */
	function &getExtra(){
		if (isset($this->_extra)) {
			return $this->_extra;
		}
	}

	/**
	 * make an element "required"
	 * 
     * @param	object  &$formElement    reference to a {@link XoopsFormElement}
	 */
	function setRequired(&$formElement){
		$this->_required[] =& $formElement;
	}

	/**
	 * get an array of "required" form elements
	 * 
     * @return	array   array of {@link XoopsFormElement}s 
	 */
	function &getRequired(){
		return $this->_required;
	}

	/**
	 * insert a break in the form
     * 
     * This method is abstract. It must be overwritten in the child classes.
	 * 
     * @param	string  $extra  extra information for the break
     * @abstract
	 */
	function insertBreak($extra = null){
	}

	/**
	 * returns renderered form
	 * 
     * This method is abstract. It must be overwritten in the child classes.
     * 
     * @abstract
	 */
	function render(){
	}

	/**
	 * displays rendered form
	 */
	function display(){
		echo $this->render();
	}

	/**
	 * assign to smarty form template instead of displaying directly
	 * 
     * @param	object  &$tpl    reference to a {@link Smarty} object
     * @see     Smarty
	 */
	function assign(&$tpl){
		$i = 0;
		foreach ( $this->getElements() as $ele ) {
			if ( !$ele->isHidden() ) {
				$elements[$i]['caption'] = $ele->getCaption();
				$elements[$i]['body'] = $ele->render();
				$elements[$i]['hidden'] = false;
			} else {
				$elements[$i]['caption'] = '';
				$elements[$i]['body'] = $ele->render();
				$elements[$i]['hidden'] = true;
			}
			$i++;
		}
		$js = "
		<!-- Start Form Vaidation JavaScript //-->
		<script type='text/javascript'>
		<!--//
		function xoopsFormValidate_".$this->getName()."(){
		";
		$required =& $this->getRequired();
		$reqcount = count($required);
		for ($i = 0; $i < $reqcount; $i++) {
			$js .= "if ( window.document.".$this->getName().".".$required[$i]->getName().".value == \"\" ) {alert( \"".sprintf(_FORM_ENTER, $required[$i]->getCaption())."\" );window.document.".$this->getName().".".$required[$i]->getName().".focus();return false;\n}
				";
		}
		$js .= "}
		//-->
		</script>
		<!-- End Form Vaidation JavaScript //-->
		";
		$tpl->assign($this->getName(), array('title' => $this->getTitle(), 'name' => $this->getName(), 'action' => $this->getAction(),  'method' => $this->getMethod(), 'extra' => 'onsubmit="return xoopsFormValidate_'.$this->getName().'();"'.$this->getExtra(), 'javascript' => $js, 'elements' => $elements));
	}
}
?>